
`default_nettype none

module top(
	input  wire Clock,				// 20 25MHz
	input  wire Reset,				// 17
	input  wire EncoderA_i,			// 66
	
	output wire Tx_o,				// 74 (Tx)
	output wire Busy_o				// 27 (SPI-CS)
);

	parameter CLOCK_HZ = 25_000_000;
	
	// Wykrywanie zbocza przycisku
	wire ManualRequest;
	Encoder EncoderA(
		.Clock(Clock),
		.Reset(Reset),
		.AsyncA_i(1'b1),
		.AsyncB_i(1'b1),
		.AsyncS_i(EncoderA_i),
		.Increment_o(),
		.Decrement_o(),
		.ButtonPress_o(ManualRequest),
		.ButtonRelease_o(),
		.ButtonState_o()		
	);
	
	
	// Message to send
	reg [7:0] Memory [0:7];
	initial begin
		Memory[0] = "H";
		Memory[1] = "e";
		Memory[2] = "l";
		Memory[3] = "l";
		Memory[4] = "o";
		Memory[5] = " ";
		Memory[6] = 8'd0;
		Memory[7] = 8'd0;
	end
	
	// Variables
	wire ByteTransmitBusy;
	wire ByteTransmitDone;
	
	// Pointer that selects a byte in memory to be transmitted next
	reg [2:0] Pointer;	
	always @(posedge Clock, negedge Reset) begin
		if(!Reset) begin
			Pointer <= 0;
		end else if(ManualRequest || ByteTransmitDone) begin
			Pointer <= Pointer + 1'b1;
		end else if(!ByteTransmitBusy) begin
			Pointer <= 0;
		end
	end
	
	wire ByteTransmitRequest = ManualRequest || (ByteTransmitDone && (Memory[Pointer] != 8'd0));
	
	// Instantiate device under test
	UartTx #(
		.CLOCK_HZ(CLOCK_HZ),
		.BAUD(115_200)
	) UartTx_inst(
		.Clock(Clock),
		.Reset(Reset),
		.Start_i(ByteTransmitRequest),
		.Data_i(Memory[Pointer]),
		.Busy_o(ByteTransmitBusy),
		.Done_o(ByteTransmitDone),
		.Tx_o(Tx_o)
	);
	
	assign Busy_o = ByteTransmitBusy;
	

endmodule

`default_nettype wire
